package NET.worlds.scape;

import java.io.IOException;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Vector;

public class Sensor
  extends SuperRoot
{
  protected Vector actions = new Vector();
  
  public void addAction(Action o)
  {
    this.actions.addElement(o);
  }
  
  public void deleteAction(Action o)
  {
    this.actions.removeElement(o);
  }
  
  public void deleteActions()
  {
    while (this.actions.size() > 0) {
      deleteAction((Action)this.actions.elementAt(0));
    }
  }
  
  public Enumeration getActions()
  {
    return this.actions.elements();
  }
  
  public int countActions()
  {
    return this.actions.size();
  }
  
  public void trigger(Event event)
  {
    RunningActionHandler.trigger(this.actions, getWorld(), event);
  }
  
  public Object properties(int index, int offset, int mode, Object value)
    throws NoSuchPropertyException
  {
    Object ret = null;
    switch (index - offset)
    {
    case 0: 
      if (mode == 0) {
        ret = ObjectPropertyAdder.make(new VectorProperty(this, index, 
          "Targets"), getRoot(), "NET.worlds.scape.Action");
      } else if (mode == 1) {
        ret = this.actions.clone();
      } else if (mode == 4) {
        deleteAction((Action)value);
      } else if (mode == 3) {
        addAction((Action)value);
      }
      break;
    default: 
      ret = super.properties(index, offset + 1, mode, value);
    }
    return ret;
  }
  
  private static Object classCookie = new Object();
  
  public void saveState(Saver s)
    throws IOException
  {
    s.saveVersion(2, classCookie);
    super.saveState(s);
    s.saveVector(this.actions);
  }
  
  public void restoreState(Restorer r)
    throws IOException, TooNewException
  {
    restoreStateVers(r);
  }
  
  protected int restoreStateVers(Restorer r)
    throws IOException, TooNewException
  {
    int vers = r.restoreVersion(classCookie);
    switch (vers)
    {
    case 1: 
    case 2: 
      super.restoreState(r);
    case 0: 
      this.actions = r.restoreVector();
      break;
    default: 
      throw new TooNewException();
    }
    return vers;
  }
  
  public void postRestore(int version)
  {
    super.postRestore(version);
    Vector newActions = new Vector();
    for (Enumeration en = getActions(); en.hasMoreElements();)
    {
      Action a = (Action)en.nextElement();
      if ((getOwner() != null) && (a.getOwner() == null))
      {
        System.out.println("Reparenting orphan action " + a.getName());
        WObject w = (WObject)getOwner();
        w.addAction(a);
      }
      if(a instanceof MoveAction) {
          MoveAction ma = (MoveAction)a;
          newActions.addAll(moveActionIntoVector(ma));
      } else {
          newActions.add(a);
      }
    }
    this.actions = newActions;
  }
  
  private Vector moveActionIntoVector(MoveAction moveAction) {
      Vector result = new Vector();
      for (Enumeration pa = moveAction.getParallelActions().elements(); pa.hasMoreElements();) {
          Action element = (Action)pa.nextElement();
          if(element instanceof MoveAction) {
              result.addAll(moveActionIntoVector((MoveAction)element));
          } else {
              result.add(element);
          }
      }
      if(moveAction.getFollowingActions().size() > 0) {
          SequenceAction sa = new SequenceAction();
          result.add(sa);
          sa.addComponent(moveAction);
          
          for (Enumeration fa = moveAction.getFollowingActions().elements(); fa.hasMoreElements();) {
              Action element = (Action)fa.nextElement();
              if(element instanceof MoveAction) {
                  MoveAction maElement = (MoveAction)element;
                  ParallelAction parallelAction = new ParallelAction();
                  parallelAction.actions.addAll(moveActionIntoVector(maElement));
                  sa.addComponent(parallelAction);
              } else {
                  sa.addComponent(element);
              }
          }
      } else {
          result.add(moveAction);
      }
      return result;
  }
}
